package grailsgtd

import org.springframework.dao.DataIntegrityViolationException

/*
 * Controller pro úkoly
 */
class TaskController {

    /* nastavení povolených http metod pro některé akce */
    static allowedMethods = [save: "POST", update: "POST", search: "POST"]

    /**
     *  Zobrazuje výchozí stránku s přehledem tagů, úkolů a jejich vyhledáváním
     *  Parametr max slouží pro stránkování
     * @param max
     * @return
     */
    def list(Integer max) {
        params.max = Math.min(max ?: 5, 10)

        // filtrování úkolů, dle aktivních tagů, které mají být zobrazeny
        def taggedTasks = [:]
        def activeTagsSize = 0

        if (session.activeTags) {
            activeTagsSize = session.activeTags.size()

            def tags = Tag.findAllByIdInList(session.activeTags)

            tags.each { tag ->
                tag.task.each { task ->
                    if (taggedTasks[task.id])
                        taggedTasks[task.id] += 1
                    else
                        taggedTasks.put(task.id, 1)
                }
            }

            taggedTasks = taggedTasks.findAll { id, count -> (count == activeTagsSize) }.keySet()

        }

        def filterParams = {
            if (taggedTasks) {
                'in'('id', taggedTasks)
            } else if (session.activeTags) {
                eq('id', 1)
                eq('id', 2)
            }

            if (session.searchParams?.priority) {
                eq('priority', session.searchParams.priority)
            }

            if (session.searchParams?.name) {
                like('name', session.searchParams.name)
            }

            if (session.searchParams?.state) {
                state {
                    eq('id', session.searchParams.state)
                }
            }

            if (session.searchParams?.todate) {
                eq('date', session.searchParams.date)
            }

            user {
                eq('username', session.user.username)
            }
        }

        // aplikace filtrů a vyhledávání
        def tasks = Task.createCriteria().list(params) { filterParams }

        [taskInstanceList: tasks, taskInstanceTotal: Task.createCriteria().count {filterParams}, tags: Tag.where { user == session.user }.list(), activeTags: session.activeTags, searchParams: session.searchParams]
    }

    /**
     * Zobrazuje formulář pro vytvoření nového úkolu
     * @return
     */
    def create() {
        [taskInstance: new Task(params)]
    }

    /**
     * Ukládá nový úkol z formuláře
     * @return
     */
    def save() {
        def taskInstance = new Task(params)
        taskInstance.user = session.user
        if (!taskInstance.save(flush: true)) {
            render(view: "/task/create", model: [taskInstance: taskInstance])
            return
        }

        flash.message = message(code: 'task.created.message')
        redirect(action: "show", id: taskInstance.id)
    }

    /**
     * Dle parametru zobrazuje úkol a komentáře
     * @param id
     * @return
     */
    def show(Long id) {
        def taskInstance = Task.get(id)
        if ((!taskInstance) || (taskInstance.user.username != session.user.username)) {
            flash.message = message(code: 'task.not.found.message')
            redirect(action: "list")
            return
        }

        [taskInstance: taskInstance]
    }

    /**
     * Dle parametru načte do editačního formuláře data pro následnou úpravu
     * @param id
     * @return
     */
    def edit(Long id) {
        def taskInstance = Task.get(id)
        if ((!taskInstance) || (taskInstance.user.username != session.user.username)) {
            flash.message = message(code: 'task.not.found.message')
            redirect(action: "list")
            return
        }

        [taskInstance: taskInstance]
    }

    /**
     * Dle parametru provede aktualizaci úkolu v databázi s kontrolou,
     * zda již mezitím nedošlo k jiné aktualizaci (druhý parametr Version)
     * @param id
     * @param version
     * @return
     */
    def update(Long id, Long version) {
        def taskInstance = Task.get(id)

        if ((!taskInstance) || (taskInstance.user.username != session.user.username)) {
            flash.message = message(code: 'task.not.found.message')
            redirect(action: "list")
            return
        }

        if (version != null) {
            if (taskInstance.version > version) {
                taskInstance.errors.rejectValue("version", "default.optimistic.locking.failure")
                render(view: "edit", model: [taskInstance: taskInstance])
                return
            }
        }

        taskInstance.properties = params

        if (!taskInstance.save(flush: true)) {
            render(view: "edit", model: [taskInstance: taskInstance])
            return
        }

        flash.message = message(code: 'task.updated.message')
        redirect(action: "show", id: taskInstance.id)
    }

    /**
     * Dle parametru provede smazání daného úkolu
     * @param id
     * @return
     */
    def delete(Long id) {
        def taskInstance = Task.get(id)
        if ((!taskInstance) || (taskInstance.user.username != session.user.username)) {
            flash.message = message(code: 'task.not.found.message')
            redirect(action: "list")
            return
        }

        try {
            taskInstance.delete(flush: true)
            flash.message = message(code: 'task.deleted.message')
            redirect(action: "list")
        }
        catch (DataIntegrityViolationException e) {
            flash.message = message(code: 'task.not.deleted.message')
            redirect(action: "show", id: id)
        }
    }

    /**
     * Uloží identifikátor tagu ve formě parametru do session pro pozdější filtrování
     * @param id
     * @return
     */
    def selectTag(Integer id) {
        def activeTags = session.activeTags;
        if (activeTags) {
            session.activeTags.add(id)
        } else {
            activeTags = []
            activeTags.add(id)
            session.activeTags = activeTags
        }

        redirect(action: "list")

    }

    /**
     * Provede vymazání daného identifikátoru ze session
     * @param id
     * @return
     */
    def deselectTag(Integer id) {
        session.activeTags.remove((Object) id)

        redirect(action: "list")
    }

    /**
     * Zpracovává filtrační formulář, resp. ukládá hodnoty do session pro
     * pozdější filtrování
     * @return
     */
    def search() {

        def conditions = [:]

        if (params.priority)
            conditions['priority'] = params.priority.toInteger()

        if (params.name)
            conditions['name'] = params.name

        if (params.state)
            conditions['state'] = params.state.toInteger()

        if (params.todate)
            conditions['todate'] = params.todate

        session.searchParams = conditions

        redirect(action: "list")
    }

    /**
     * Parametry ze session pro vyhledávání vymaže
     * @return
     */
    def unsetSearchParams() {
        session.removeAttribute('searchParams')

        redirect(action: "list")
    }
}
