<?php

use \Nette\Application\UI\Form, \Models\Task;

/**
 * Task presenter.
 */
class TaskPresenter extends BasePresenter
{

    /** @persistent array */
    public $activeTags = array();

    /** @persistent array */
    public $searchParams = array();

    /** @var \Models\Task */
    protected $taskModel;

    /** @var \Models\Tag */
    protected $tagModel;

    /** @var \Models\State */
    protected $stateModel;

    /** @var \Models\TaskTag */
    protected $taskTagModel;

    /** @var \Models\Comment */
    protected $commentModel;


    /**
     * Injektáž modelů
     * @param Models\Task $taskModel
     * @param Models\Tag $tagModel
     * @param Models\State $stateModel
     * @param Models\TaskTag $taskTagModel
     * @param Models\Comment $commentModel
     * @throws Nette\InvalidStateException
     */
    public function injectModels(\Models\Task $taskModel, \Models\Tag $tagModel, \Models\State $stateModel, \Models\TaskTag $taskTagModel, \Models\Comment $commentModel)
    {
        if (($this->taskModel) || ($this->tagModel) || ($this->stateModel) || ($this->taskTagModel) || ($this->commentModel)) {
            throw new Nette\InvalidStateException('Injektáž již proběhla.');
        }

        $this->taskModel = $taskModel;
        $this->tagModel = $tagModel;
        $this->stateModel = $stateModel;
        $this->taskTagModel = $taskTagModel;
        $this->commentModel = $commentModel;
    }

    /**
     * Zobrazení detailu úkolu
     * @param $id
     */
    public function renderView($id)
    {
        if (!$id) {
            $this->error('Neplatný identifikátor');
        }

        $task = $this->taskModel->fetch($id, $this->getUser()->getId());

        if (!$task) {
            $this->error('Neplatný identifikátor');
        }


        $this->template->task = $task;
        $this->template->tags = $this->taskTagModel->fetchAllByTask($id);
    }

    /**
     * Zobrazení editace s využitím formuláře a view pro přidávání úkolů
     * @param $id
     */
    public function renderEdit($id)
    {
        if (!$id) {
            $this->error('Neplatný identifikátor');
        }

        // proveď pouze pokud je formulář odeslán
        if (!$this['taskForm']->isSubmitted()) {
            $task = $this->taskModel->fetch($id, $this->getUser()->getId());

            if(!$task){
                $this->error('Neplatný identifikátor');
            }

            // nastavení výchozích hodnot
            $this['taskForm']->setDefaults($task);

            if(isset($task->todate))
                $this['taskForm']['todate']->setValue(\Nette\Templating\Helpers::date($task->todate, 'j.n.Y'));

            // načtení tagů do multiselectu
            $this['taskForm']['tags']->setValue($this->taskTagModel->fetchAllByTask($id));
        }

        $this->setView('add');
    }


    /**
     * Přehled všech úkolů s filtrováním, výpisem tagů atd..
     */
    public function renderDefault()
    {
        // pole filtrovacích podmínek
        $conditions = array();

        // vybere všechny úkoly, které odpovídají zadaným tagům
        if (!empty($this->activeTags)) {
            $tags = $this->taskTagModel->fetchTaskByActiveTags($this->activeTags);

            $cond = array();
            foreach ($tags as $tag)
                $cond[] = $tag->task_id;

            $conditions = array('id' => $cond);
        }

        // omezení na aktuální uživatele
        $conditions['user_email'] = $this->getUser()->getId();

        // spojení podmínek pro filtrování a selekci na základě tagů
        $conditions = array_merge($this->searchParams, $conditions);

        $items = $this->taskModel->fetchAll()->where($conditions);

        // inicializace vizuálního stránkovače pomocí komponenty
        $paginator = $this['visualPaginator']->getPaginator();
        $paginator->itemCount = $items->count();

        $items = $items->limit($paginator->itemsPerPage, $paginator->offset);


        // nastavení formuláře pro vyhledávání, hodnoty pro sloupec tagů

        $this->template->activeTags = $this->activeTags;
        $this->template->tags = $this->tagModel->fetchPairs('id', 'name', $this->getUser()->getId());

        $this->template->tasks = $items;
    }

    /**
     * Tovřánička na stránkovač
     * @param $name
     * @return VisualPaginator
     */
    protected function createComponentVisualPaginator($name){
        $vp = new \VisualPaginator($this, $name);
        $vp->getPaginator()->itemsPerPage = 10;
        return $vp;
    }


    /**
     * Továrna formuláře pro úpravu úkolu
     * @return Nette\Application\UI\Form
     */
    protected function createComponentTaskForm()
    {
        $form = new Form();

        $form->addText('name', 'Název:')
            ->setRequired('Vyplňte název úkolu');

        $form->addTextArea('description', 'Popis:');

        $form->addSelect('priority', 'Priorita:', Task::$priorities)
            ->setPrompt('Vyberte prioritu')
            ->setRequired('Vyberte prioritu');


        $form->addText('todate', 'Do kdy:')
            ->addCondition(Form::FILLED, true)
            ->addRule(
            Form::PATTERN,
            'Neplatný formát data',
            '^([1-9]|19|[12][0-8]|29(?=\.([^2]|2\.(([02468][048]|[13579][26])00|[0-9]{2}(0[48]|[2468][048]|[13579][26]))))|30(?=\.[^2])|31(?=\.([13578][02]?\.)))\.([1-9]|1[012])\.[0-9]{4}$'
        );

        $form->addMultiSelect('tags','Tagy:',
            $this->tagModel->fetchPairs('id','name',$this->getUser()->getId())
        );

        $form->addSelect('state_id', 'Stav', $this->stateModel->fetchPairs('id', 'name'))
            ->setPrompt('Vyberte stav')
            ->setRequired('Vyberte stav');


        $form->addSubmit('send', 'Uložit');

        $form->onSuccess[] = $this->taskFormSucceeded;
        return $form;
    }


    /**
     * Callback pro odeslaný formulář s úkolem
     * @param $form
     */
    public function taskFormSucceeded($form)
    {
        if ($form->isSubmitted()) {
            $id = (int)$this->getParameter('id');

            $values = $form->getValues();

            // extrakce hodnot pro tagy a datum
            $tags = $values['tags'];
            unset($values['tags']);

            if ($values['todate'])
                $values['todate'] = new \Nette\DateTime($values['todate']);
            else
                unset($values['todate']);


            // proveď update, pokud se jedná o editaci, jinak inser
            if ($task = $this->taskModel->fetch($id, $this->getUser()->getId())) {
                $task->update($values);
            } else {
                $values['user_email'] = $this->getUser()->getId();

                $row = $this->taskModel->insert($values);
                $id = $row->id;
            }

            // zpracování tagů
            $this->taskTagModel->updateAll($id, $tags);

            $this->flashMessage('Úkol byl úspěšně uložen.', 'success');
            $this->redirect('default');

        }
    }

    /**
     * Továrna formuláře pro hledání
     * @return Nette\Application\UI\Form
     */
    protected function createComponentSearchForm()
    {
        $form = new Form();

        $form->addText('name', 'Název:');

        $form->addSelect('state_id', 'Stav', $this->stateModel->fetchPairs('id', 'name'))
            ->setPrompt('Vyberte stav');

        $form->addSelect('priority', 'Priorita:', Task::$priorities)
            ->setPrompt('Vyberte prioritu');

        $form->addText('todate', 'Do kdy:')
            ->addCondition(Form::FILLED, true)
            ->addRule(
            Form::PATTERN,
            'Neplatný formát data',
            '^([1-9]|19|[12][0-8]|29(?=\.([^2]|2\.(([02468][048]|[13579][26])00|[0-9]{2}(0[48]|[2468][048]|[13579][26]))))|30(?=\.[^2])|31(?=\.([13578][02]?\.)))\.([1-9]|1[012])\.[0-9]{4}$'
        );

        $form->addSubmit('search', 'Vyhledat');

        $form->onSuccess[] = $this->searchFormSucceeded;
        return $form;
    }

    /**
     * Callback pro odeslaný formulář s vyhledávacími parametry
     * Provede postupné uložení všech parametrů do persistentního pole
     * @param $form
     */
    public function searchFormSucceeded($form)
    {
        if ($form->isSubmitted()) {
            foreach ($form->getValues() as $key => $value) {
                if (($value) || ($value == "0")) {
                    if ($key == 'todate')
                        $this->searchParams[$key] = new \Nette\DateTime($value);
                    else
                        $this->searchParams[$key] = $value;
                } else {
                    unset($this->searchParams[$key]);
                }

            }
        }
    }


    /**
     * Uloží daný tag jako parametr pro filtrování
     * @param $id
     */
    public function handleSelectTag($id)
    {
        if (!$id) {
            return;
        }

        $this->activeTags[] = $id;
    }

    /**
     * Odstranění tag z pole parametrů pro vyhledávání
     * @param $id
     */
    public function handleDeselectTag($id)
    {
        if (!$id) {
            return;
        }

        $position = array_search($id, $this->activeTags);
        if ($position !== false)
            unset($this->activeTags[$position]);
    }

    /**
     * Odstranění veškeré nastavené filtry
     */
    public function handleDeleteSearchParams()
    {
        $this->searchParams = array();
    }

    /**
     * Smaže úkol dle id
     * @param $id
     */
    public function actionDelete($id)
    {
        if (!$id) {
            $this->error('Neplatný identifikátor');
        }

        if ($this->taskModel->delete($id, $this->getUser()->getId())) {
            $this->flashMessage('Úkol byl úspěšně smazán.', 'success');
        } else {
            $this->flashMessage('Úkol se nepodařilo smazat.', 'error');
        }

        $this->redirect('default');
    }


    /**
     * Továrna formuláře pro tvorbu komentáře
     * @return Nette\Application\UI\Form
     */
    protected function createComponentCommentForm()
    {
        return new CommentForm($this->commentModel, $this->taskModel);
    }
}
