<?php

namespace Test;

use Nette,
    Tester,
    Tester\Assert;

$container = require __DIR__ . '/bootstrap.php';


/**
 * Testování TaskPresenteru v přihlášeném stavu
 */
class TaskPresenterLoggedTest extends Tester\TestCase
{
    /**
     * @var \Nette\DI\Container
     */
    private $container;
    /**
     * @var identifikátor testovaného záznamu
     */
    private $id;

    /**
     * @var \TaskPresenter
     */
    private $presenter;

    /**
     * @param \Nette\DI\Container $container
     */
    function __construct(Nette\DI\Container $container)
    {
        $this->container = $container;
    }


    /**
     * Přípravná fáze, vytvoření presenteru, injekce a přihlášení
     */
    function setUp()
    {
        $this->presenter = new \TaskPresenter($this->container);
        $this->presenter->injectModels(
            $this->container->getService('taskModel'),
            $this->container->getService('tagModel'),
            $this->container->getService('stateModel'),
            $this->container->getService('taskTagModel'),
            $this->container->getService('commentModel')
        );

        $this->login();
    }

    /**
     * Přihlášení testovacího uživatele
     */
    private function login(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test@test.cz',
                'password' => 'test1'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }


    /**
     * Přihlášení testovacího uživatele
     */
    private function loginAnother(){
        $request = new \Nette\Application\Request('User','POST',
            array(
                'action' => 'in',
                'do' => 'signInForm-submit'
            ),
            array(
                'email' => 'test1@test.cz',
                'password' => 'password'
            )
        );
        $this->container->createInstance('UserPresenter')->run($request);

    }






    /**
     * Signál pro výběr tagu, otestování vložené hodnoty
     */
    public function testSelectTag(){
        $id = 1;

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'id' => $id ,
                'do' => 'selectTag'
            )
        );
        $response = $this->presenter->run($request);

        $position = array_search($id, $response->getSource()->presenter->activeTags);

        Assert::true($position !== FALSE);
    }

    /**
     * Signál pro odebrání tagu z filtru a otestování, zda se skutečně odebral
     */
    public function testDeselectTag(){
        $id = 1;

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'id' => $id ,
                'do' => 'deselectTag'
            )
        );
        $response = $this->presenter->run($request);

        $position = array_search($id, $response->getSource()->presenter->activeTags);

        Assert::false($position);
    }

    /**
     * Testování přítomnosti formuláře pro přidání úkolu
     */
    public function testAddTaskForm(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'add',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response->getSource()->presenter['taskForm'] instanceof \Nette\Application\UI\Form);
    }

    /**
     * Testování funkčnosti formuláře pro přidání úkolu
     */
    public function testAdd(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'add',
                'do' => 'taskForm-submit'
            ),
            array(
                'name' => 'Pokus',
                'description' => 'Pokus',
                'priority' => '1',
                'state_id' => '1',
                'description' => 'Pokus',
                'todate' => '1.1.2012',
            )
        );

        $response = $this->presenter->run($request);

        $this->id = $this->container->getService('taskModel')->fetchAll()->order('id DESC')->fetch()->id;

        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);
    }

    /**
     * Testování reakce formuláře pro přidání úkolu na chyby
     */
    public function testAddFailure(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'add',
                'do' => 'taskForm-submit'
            ),
            array(
                'name' => '',
                'description' => 'Pokus',
                'priority' => 'Pokus',
                'state_id' => 'Pokus',
                'description' => 'Pokus',
                'todate' => '1.1.2012',
                'tags' => ''
            )
        );

       $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }

    /**
     * Testování zobrazení vloženého úkolu a formuláře pro komentáře
     */
    public function testView(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
                'id' => $this->id
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
        Assert::true($response->getSource()->task instanceof \Nette\Database\Table\ActiveRow);

        Assert::true($response->getSource()->presenter['commentForm'] instanceof \Nette\Application\UI\Form);
    }

    /**
     * Testování zobrazení vloženého úkolu a formuláře pro komentáře při neautozizovaném přístupu
     */
    public function testViewNotAuthorized(){
        $this->loginAnother();

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
                'id' => $this->id
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }


    /**
     * Testování reakce formuláře pro přidání komentáře s chybou
     */
    public function testCommentAddFailure(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
                'id' => $this->id,
                'do' => 'commentForm-submit'
            ),
            array(
                'text' => '',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }



    /**
     * Testování reakce formuláře pro přidání komentáře se správně vyplněným vstupem
     */
    public function testCommentAdd(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
                'id' => $this->id,
                'do' => 'commentForm-submit'
            ),
            array(
                'text' => 'Pokus',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);
        Assert::true($this->container->getService('commentModel')->count() > 0);
    }

    /**
     * Testování reakce formuláře pro přidání komentáře se správně vyplněným vstupem ale neautorizovaně
     */
    public function testCommentAddNotAuthorized(){
        $this->loginAnother();

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
                'id' => $this->id,
                'do' => 'commentForm-submit'
            ),
            array(
                'text' => 'Pokus',
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }

    /**
     * Testování reakce na neexistující parametr id při zobrazení úkolu
     */
    public function testViewFailure(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'view',
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }

    /**
     * Testování reakce na neexistující parametr id při editaci úkolu
     */
    public function testEditFailure(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'edit',
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }

    /**
     * Testování přítomnosti formuláře při editaci úkolu
     */
    public function testEditForm(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'edit',
                'id' => $this->id
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response->getSource()->presenter['taskForm'] instanceof \Nette\Application\UI\Form);
    }

    /**
     * Testování přítomnosti formuláře pro editaci úkolu při neautorizovaném přístupu
     */
    public function testEditFormNotAuthorized(){
        $this->loginAnother();

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'edit',
                'id' => $this->id
            )
        );

        try{
            $this->presenter->run($request);
        }catch (\Exception $e){
        }

        Assert::true($e instanceof Nette\Application\BadRequestException);
    }

    /**
     * Testování reakce při chybném vyplnění formuláře pro editaci
     */
    public function testEditSendFailure(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'edit',
                'id' => $this->id,
                'do' => 'taskForm-submit'
            ),
            array(
                'name' => '',
                'description' => 'Pokus',
                'priority' => 'Pokus',
                'state_id' => 'Pokus',
                'description' => 'Pokus',
                'todate' => '1.1.2012',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }

    /**
     * Testování formuláře při správném vyplnění při editaci úkolu
     */
    public function testEditSend(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'edit',
                'id' => $this->id,
                'do' => 'taskForm-submit'
            ),
            array(
                'name' => 'Pokus',
                'description' => 'Pokus',
                'priority' => '1',
                'state_id' => '1',
                'description' => 'Pokus',
                'todate' => '1.1.2012',
                'tags' => 'pokus'
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);
    }


    /**
     * Testování signálu pro smazání úkolu
     */
    public function testDelete(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'delete',
                'id' => $this->id
            )
        );

        $response = $this->presenter->run($request);
        Assert::true($response instanceof \Nette\Application\Responses\RedirectResponse);

        Assert::false($this->container->getService('taskModel')->fetch($this->id));
    }

    /**
     * Testování přítomnosti vyhledávacího formuláře v přehledu úkolů
     */
    public function testSearchForm(){
        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
            )
        );

        $response = $this->presenter->run($request);

        Assert::true($response->getSource()->presenter['searchForm'] instanceof \Nette\Application\UI\Form);

    }

    /**
     * Testování funkce vyhledat, resp. zda dojde k uložení hodnot do persistentního parametru
     * při odeslání formuláře.
     */
    public function testSearch(){
        $params = array(
            'name' => 'Test',
            'priority' => '1',
            'state_id' => '1',
            'todate' => '1.1.2014',
        );

        $request = new \Nette\Application\Request('Task','POST',
            array(
                'action' => 'default',
                'do' => 'searchForm-submit'
            ),
            $params
        );

        $response = $this->presenter->run($request);

        Assert::compare($response->getSource()->presenter->searchParams['name'], $params['name']);
        Assert::compare($response->getSource()->presenter->searchParams['priority'], $params['priority']);
        Assert::compare($response->getSource()->presenter->searchParams['state_id'], $params['state_id']);
        Assert::compare($response->getSource()->presenter->searchParams['todate'], $params['todate']);

        Assert::true($response instanceof \Nette\Application\Responses\TextResponse);
    }
}


id(new TaskPresenterLoggedTest($container))->run();